#! /usr/bin/gawk -f
# ######################################################################### #
#                                                                           #
# File:     LeMonde2Text.awk                                                #
# Authors:  Brigitte Bigi                                                   #
# Date:     november, 2003 (last modif: 26/02/2004)                         #
# Licence:  GPL                                                             #
# Brief:     Programme de filtrage des corpus du journal "Le Monde" de 1987 #
#             2002 (corpus fournit par ELRA).                              #
#            Voir le fichier corpus.dtd pour plus d'infos sur les tags      #
#                                                                           #
# ######################################################################### #


BEGIN{
	# Booleen : dans un article ?
	body = -1	

	# Theme de l article
	subject = 0

	# Ecriture de l'entete xml
	print "\n<corpus>\n"
}


# -------------------------
# Debut d'un nouvel article
# -------------------------
/^#ACC/{
	if ( body == 1 )
	{
		print "\n</text>"
		print "</doc>"
	}
	else
	if ( body == 0 )
	{
		# Certains documents n'ont pas d'article, seulement une description
		print "\n</descr>"
		print "</doc>"
	}
	print "<doc>"
	print "<descr>"
	body = 0 
	descr= 1
}

# ----------------------------------
# Indication des thmes d'un article
# ----------------------------------
/^#PE2/{
	if ( body == 0 )
	{
		gsub(/\r/, "")
		sub(/#PE2\(1\)=/, "")

		# le theme peut etre sur 2 lignes
		theme = $0
		getline
		if (!match($0,/^#/)) 
		{
			theme = sprintf("%s%s", theme,$0)
		}

		gsub(/,/, "\n", theme)
		TAB_Subject[subject] = theme
		subject ++
	}
}
/^#ET2/{
	if ( body == 0 )
	{
		gsub(/\r/, "")
		sub(/#ET2\(1\)=/, "")

		# le theme peut etre sur 2 lignes
		theme = $0
		getline
		if (!match($0,/^#/))
		{
			theme = sprintf("%s%s", theme,$0)
		}

		gsub(/,/, "\n", theme)
		TAB_Subject[subject] = theme
		subject ++
	}
}

# --------------------------------
# Indication du titre d'un article
# --------------------------------
/^#TIJ/{
	if ( body == 0 )
	{
		gsub(/\r/, "")
		gsub(/</, " ")
		gsub(/>/, " ")
		sub(/#TIJ\(1\)=/, "")
		print "<title>"
		print
		print "</title>"
	}
}

# ----------------------------------
# Indication de l'auteur d'un article
# ----------------------------------
/^#SIP/{
	if ( body == 0 )
	{
		gsub(/\r/, "")
		sub(/#SIP\(1\)=/, "")
		gsub(/,/, "\n")
		print "<creator>"
		print
		print "</creator>"
	}
}

# -----------------------------------------
# Indication de la date de l'article
# -----------------------------------------
/^#DAT/ {
	if ( body == 0 )
	{
		sub(/#DAT\(1\)=/, "")
		year  = substr($0, 1, 4)
		month = substr($0, 5, 2)
		day   = substr($0, 7, 2)
		printf "<date y=\"%s\" m=\"%s\" d=\"%s\" />\n", year, month, day
	}
}

# -----------------------------------------
# Indication des mots cles de l'article 
# -----------------------------------------
/^#FR2/ {
	if ( body == 0 )
	{
		gsub(/\r/, "")
		sub(/#FR2\(1\)=/, "")
		gsub(/,/, " </k>\n<k> ")	# <k> mot-cle </k>
		##gsub(/ /, "_")
		print "<keys>"
		print "<k>", $0, "</k>"
		print "</keys>"
	}
}

# -----------------------------------------
# Indication du dbut du corps d'un article
# -----------------------------------------
/^#TEX/ {
	gsub(/\r/, "")
	sub(/#TEX=/, "")
	if (subject != 0)
	{
		print "<subject>"
		for (i=0; i<subject; i++)
			print TAB_Subject[i]
		print "</subject>"
		subject = 0
		delete TAB_Subject
	}
	print "</descr>"
	print "<text>"
	print "<><>"
	body = 1
}

# ----------
# Autres cas
# ----------
{
	gsub (/<?#FIELD/, "")
	gsub (/<?#LINE/, "")
	gsub (/@ ARTICLE/, "")
	gsub (/@ ARTICL/, "")

 	# ------------------
	# Corps d'un article
	# ------------------
	if (body == 1)
	{
		# Marque de fin d'article
		gsub(/<[^>]/, "")

		# Retour charriot windows
		gsub(/\r/, "")

		# Marque de fin de ligne
		if (match($0,/[^<]>/) != 0 || match($0,/<>/) != 0)
		{
			$0 = gensub(/([a-zA-Z])>([a-zA-Z])/, "\\1 \\2", "g", $0)
			$0 = gensub(/([^<])>/, "\\1", "g", $0)
			gsub(/<>/, "\n<><>")
			print
		}
		else
		{
			gsub(/http :/, "http:")
			gsub(/</, " ")
			gsub(/>/, " ")
			gsub(/\n/, "")
			
			# Ecrit en evitant les coupures de mots
			if (match($0, /[a-zA-Z]+\.$/))
				printf "%s <>\n",$0
			else
				printf "%s", $0
		}
	}
}


END{
	print "\n</text>"
	print "</doc>"

	print "\n</corpus>"
}
