#! /usr/bin/gawk -f
# ######################################################################### #
#                                                                           #
# File:     Lower.awk                                                       #
# Authors:  Brigitte Bigi                                                   #
# Date:     Nov, 2005                                                       #
# Modif:    Viet-Bac LE, Sept. 2007                                         #
# Licence:  GPL                                                             #
# Brief:    Convert a text in lower case                                    #
# Help:     Lower.awk -h                                                    #
#                                                                           #
# ######################################################################### #


# ######################################################################### #
#                                                                           #
#  Function FCT_Load                                                        #
#                                                                           #
#  Description: Load 2 columns from a file                                  #
#                                                                           #
#  Parameters:                                                              #
#       file is the file name                                               #
#                                                                           #
#  Output:                                                                  #
#       TAB_Replace                                                         #
#                                                                           #
#  Return value:                                                            #
#       integer value: the number of entries in TAB_Replace                 #
#                                                                           #
# ######################################################################### #
function FCT_Load ( file )
{
        _ind = 0
        while ( getline <  file  )
        {
                TAB_Replace[$1]=$2
                _ind++
        }
        return _ind
}


# ######################################################################### #
#                                                                           #
# Function Name: FCT_Usage                                                  #
#                                                                           #
# Description:                                                              #
#     Print the usage and exit the program                                  #
#                                                                           #
# ######################################################################### #
function FCT_Usage ()
{
        printf "Usage: Lower.awk [options]\n\n"
        printf "Where options include:\n"
	printf "  -o filename  write the output to this file (default: stdout)\n"
	printf "  -V filename  file contains upper/lower character couples\n"
	printf "  -e id        make change only on elts with this id\n"
        printf "  -h	       print this help\n"
	printf "\n"
	printf " input text:  stdin\n"
	printf " output text: stdout (except if -o)\n"
        exit
}



# ######################################################################### #
#                                                                           #
# Function Name: FCT_CommandLine                                            #
#                                                                           #
# Description:                                                              #
#     Get arguments from the command line                                   #
#                                                                           #
# ######################################################################### #
function FCT_CommandLine(argc, argv)
{
        for (i=1; i<argc; i++)
        {
                if (argv[i] == "-h")
                        FCT_Usage()
                else
		if (argv[i] == "-e")
		{
			EltsID = argv[i+1]
			i++
		}
                else
		if (argv[i] == "-V")
		{
			tabsize = FCT_Load(argv[i+1])
			i++
		}
		else
                if (argv[i] == "-o")
                {
                        Output = argv[i+1]
                        i++
                }
                else
		if (argv[i] ~ /^\-/)
		{
			printf "invalid option -- %s\n", argv[i] > "/dev/stderr"
			FCT_Usage()
		}
        }
}


# ############################# BEGIN MAIN ################################ #

BEGIN {
	EltsID      = "NULL"		# elts id to lower
	InElts      = "TRUE"
	Output      = "/dev/stdout"	# output file name
	tabsize     = 0
	FCT_CommandLine(ARGC, ARGV)	# extract parameters
        delete ARGV
        ARGC=1
}

/<elts[ >]/ {
	InElts="TRUE"
	if (EltsID != "NULL")
	{
		# Get elts id
	        elts=$0
		gsub(/>/, " ",elts)
		s = index(elts, "id")				# start: id
	        e = index(substr(elts, s+3, length(elts)), " ")	# end: 1st space after s
        	id = substr(elts, s+3, e-1)			# id: $0 from s to e
	        gsub(/ /, "", id)				# remove spaces
 		# and compare with the given id
		if (id!=EltsID) InElts="FALSE"
	}
}

/<str[ >]/ || /<lem[ >]/ {
	if (InElts=="TRUE")
	{
	        # Get the word
        	s = index($0, ">")				# start: 1st >
	        e = index(substr($0, s+1, length($0)), "<")	# end: 1st < after start
        	w = substr($0, s+1, e-1)			# word: $0 from start to end
	        gsub(/ /, "", w)				# remove spaces

		# Convert and print
		w = tolower(w)			# lower 0-128 letters
		for (c in TAB_Replace)		# replace upper case by lower case using a mapping table
		{
			if (match(w, c))
			gsub(c, TAB_Replace[c], w)
		}
	}							
}

/<str[ >]/ {
	if (InElts=="TRUE")
	{
		print "<str>",w,"</str>" > Output
		next
	}
}

/<lem[ >]/ {
	if (InElts=="TRUE")
	{
		print "<lem>",w,"</lem>" > Output
		next
	}
}

{
	print > Output
}

