#! /usr/bin/gawk -f
# ######################################################################### #
#                                                                           #
# File:     Num2LetterEN.awk                                                #
# Authors:  Brigitte Bigi                                                   #
# Date:     August, 2008                                                    #
# Licence:  GPL                                                             #
# Brief:    Convert numbers of a corpus to the corresponding string (EN)    #
# Help:     Num2LetterEN.awk -h                                             #
#                                                                           #
# ######################################################################### #



# ######################################################################### #
#                                                                           #
# Function: unite                                                           #
# 	Converts numbers from 0 to 9 and returns the corresponding string   #
#                                                                           #
# ######################################################################### #
function unite( number )
{
	_r = ""
	if (number == 9)  _r = "nine "
	if (number == 8)  _r = "height "
	if (number == 7)  _r = "seven "
	if (number == 6)  _r = "six "
	if (number == 5)  _r = "five "
	if (number == 4)  _r = "four "
	if (number == 3)  _r = "three "
	if (number == 2)  _r = "two "
	if (number == 1)  _r = "one "
	if (number == 0)
	{
		_str = " zero "
		for(i=1; i<length(number); i++)
			_str = " " _str " zero "
		return _str
	}
	return _r
}


# ######################################################################### #
#                                                                           #
# Function: dizaine                                                         #
# 	Converts numbers from 1 to 99 and returns the corresponding string  #
#                                                                           #
# ######################################################################### #
function dizaine( number )
{
	if (number == 90)
		_str = "ninety "
	else if (number > 90 && number <= 99)
		_str = "ninety-" dizaine(number-90)
	else if (number == 80)
		_str = "heighty "
	else if (number > 80 && number <= 89)
		_str = "heighty-" dizaine(number-80)
	else if (number == 70)
		_str = "seventy "
	else if (number > 70 && number <= 79)
		_str = "seventy-" dizaine(number-70)
	else if (number == 60)
		_str = "sixty "
	else if (number > 60 && number <= 69)
		_str = "sixty-" dizaine(number-60)
	else if (number == 50)
		_str = "fifty "
	else if (number > 50 && number <= 59)
		_str = "fifty-" dizaine(number-50)
	else if (number == 40)
		_str = "fourty "
	else if (number > 40 && number <= 49)
		_str = "fourty-" dizaine(number-40)
	else if (number == 30)
		_str = "thirty "
	else if (number > 30 && number <= 39)
		_str = "thirty-" dizaine(number-30)
	else if (number == 20)
		_str = "twenty "
	else if (number > 20 && number <= 29)
		_str = "twenty-" dizaine(number-20)
	else if (number >= 10 && number <= 19)
	{
		if (number == 10) _r = "ten "
		if (number == 11) _r = "eleven "
		if (number == 12) _r = "twelve "
		if (number == 13) _r = "thirteen "
		if (number == 14) _r = "fourteen "
		if (number == 15) _r = "fifteen "
		if (number == 16) _r = "sixteen "
		if (number == 17) _r = "seventeen "
		if (number == 18) _r = "heigteen"
		if (number == 19) _r = "nineteen "
		return _r
	}
	else
		_str = unite(number)

	return _str
}	


# ######################################################################### #
#                                                                           #
# Function: centaine                                                        #
# 	Converts numbers from 1 to 999 and returns the corresponding string #
#                                                                           #
# ######################################################################### #
function centaine( number )
{
	if (number >= 100)
	{
		if (number > 900 && number <= 999)
			_str =  "nine hundred " dizaine(number-900)
		else if (number == 900)
			_str = "nine hundred"
		else if (number > 800 && number <= 899)
			_str =  "height hundred " dizaine(number-800)
		else if (number == 800)
			_str = "height hundred"
		else if (number > 700 && number <= 799)
			_str =  "seven hundred " dizaine(number-700)
		else if (number == 700)
			_str = "seven hundred"
		else if (number > 600 && number <= 699)
			_str =  "six hundred " dizaine(number-600)
		else if (number == 600)
			_str = "six hundred"
		else if (number > 500 && number <= 599)
			_str =  "five hundred " dizaine(number-500)
		else if (number == 500)
			_str = "five hundred"
		else if (number > 400 && number <= 499)
			_str =  "four hundred " dizaine(number-400)
		else if (number == 400)
			_str = "four hundred"
		else if (number > 300 && number <= 399)
			_str =  "three hundred " dizaine(number-300)
		else if (number == 300)
			_str = "three hundred"
		else if (number > 200 && number <= 299)
			_str =  "two hundred " dizaine(number-200)
		else if (number == 200)
			_str = "two hundred"
		else if (number > 100 && number <= 199)
			_str =  "one hundred " dizaine(number-100)
		else if (number == 100)
			_str = "one hundred"
	}
	else
		_str = dizaine(number)
		
	return _str	
}	


# ######################################################################### #
#                                                                           #
# Function: millier                                                         #
# Converts numbers from 1000 to 99999 and returns the corresponding string  #
#                                                                           #
# ######################################################################### #
function millier( number )
{
	if (number < 1000) _str = centaine(number)

	# Milliers
	# --------
	if ( number == 1000 )
		_str = "one thousand "
	
	if ( ( number >= 1000 ) && ( number < 10000 ) )
	{
		if ( (number%1000) != 0 )
			_str = unite(int (number/1000)) " thousand " centaine(number%1000)
		else
			_str = unite(int (number/1000)) " thousand "
	}


	# Dizaines de milliers
	# --------------------
	if (number == 10000)
		_str = "ten thousand "
	
	if (number > 10000 && number < 100000)
	{
		if ( (number%1000) != 0 )
			_str = dizaine(int (number/1000)) " thousand " centaine(number%1000)
		else
			_str = dizaine(int (number/1000)) " thousand"
	}
	

	# Centaines de milliers
	# ---------------------
	if (number == 100000)
		_str = "hundred thousand"
	
	if (number >= 100000 && number < 1000000)
	{
		if ( (number%1000) != 0 )
			_str = centaine(int (number/1000)) "thousand " centaine(int (number%1000))
		else
			_str = centaine(int (number/1000)) "thousand "
	}

	return _str
}


# ######################################################################### #
#                                                                           #
# Function: FCT_Num2text                                                    #
#	Converts all numbers and returns the corresponding string           #
#                                                                           #
#  number is the number to convert                                          #
#                                                                           #
# ######################################################################### #
function FCT_Num2text( number )
{
	if (number < 1000000)
		_str = millier(number)

	# Millions
	# --------
	if (number >= 1000000 && number < 2000000)
		_str = "one million " millier(int (number%1000000))

	if (number >= 2000000 && number < 10000000)
		_str = unite(int (number/1000000)) "million " millier(int (number%1000000))


	# Dizaines de millions
	# --------------------
	if (number == 10000000)
		_str = "ten million"

	if (number > 10000000 && number < 100000000)
		_str = dizaine(int (number/1000000)) "million " millier(int (number%1000000))
		

	# Centaines de millions
	# ---------------------
	if (number == 100000000)
		_str = "hundred million"

	if (number > 100000000 && number < 1000000000)
		_str = centaine(int (number/1000000)) "million " millier(int (number%1000000))

	if (number > 1000000000)
		return number

	return _str
}


# ######################################################################### #
#                                                                           #
# Function Name: FCT_Usage                                                  #
#                                                                           #
# Description:                                                              #
#     Print the usage and exit the program                                  #
#                                                                           #
# ######################################################################### #
function FCT_Usage ()
{
        printf "Usage: Num2LetterEN.awk [options]\n\n"
        printf "Where options include:\n"
	printf "  -o filename  write the output to this file (default: stdout)\n"
	printf "  -noxml       input is one word a line (instead of XML)\n"
        printf "  -h	print  this help\n"
	printf "\n"
	printf " input number: stdin\n"
	printf " output text:  stdout\n"
        exit
}


# ######################################################################### #
#                                                                           #
# Function Name: FCT_CommandLine                                            #
#                                                                           #
# Description:                                                              #
#     Get arguments from the command line                                   #
#                                                                           #
# ######################################################################### #
function FCT_CommandLine(argc, argv)
{
        entry = 0
        for (i=1; i<argc; i++)
        {
                if (argv[i] == "-h")
                        FCT_Usage()
                else
                if (argv[i] == "-noxml")
                {
                        xml = 0
                        i++
                }
                else
                if (argv[i] == "-o")
                {
                        Output = argv[i+1]
                        i++
                }
                else
		if (argv[i] ~ /^\-/)
		{
			printf "invalid option -- %s\n", argv[i] > "/dev/stderr"
			FCT_Usage()
		}
        }
}

# ######################################################################### #
#                                                                           #
# Function Name: FCT_Convert                                                #
#                                                                           #
# Description:                                                              #
#     Get a number and return a string                                      #
#                                                                           #
# ######################################################################### #

function FCT_Convert ( w )
{
	_w = w
	_strnum = ""

	# Numbers starting by one or more "0"
	if (match(_w,/^0/))
	{
		while (match(_w,/^0/))
		{
			_strnum = _strnum " zero "
			sub(/^0/, "", _w)
		}
	}
	gsub(/\./, "",_w)
	gsub(/,/, "",_w)
	if (length(_w) > 0)
	{
		_num = (int(_w))
		if (_num > 1000 && _num < 2000)
		{
			_right = _num%100
			_left = (_num-_right)/100
			_strnum = _strnum FCT_Num2text(_left) " " FCT_Num2text(_right)
		}
		else
			_strnum = _strnum FCT_Num2text(_num)
	}
	return _strnum
}



# ############################# BEGIN MAIN ################################ #

BEGIN {
	Output      = "/dev/stdout"	# output file name
	xml         = 1			# xml input ??? (yes is default)
        FCT_CommandLine(ARGC, ARGV)	# extract parameters
        delete ARGV
        ARGC=1
}

/<str>/ {
        # Get the element
        s = index($0, ">")				# start: 1st >
        e = index(substr($0, s+1, length($0)), "<")	# end: 1st < after start
        w = substr($0, s+1, e-1)			# word: $0 from start to end
        gsub(/ /, "", w)				# remove spaces

	# Verify if the input string is a number
	nombre = w
	sub(/[0-9]*/, "", nombre)

	# Num2Text only numbers! 
	if (length(nombre) == 0)
	{
		strnum = FCT_Convert(w)

		# Format the output (see dtd!)
		gsub("[  ]+", " ", strnum)	# remove multiple spaces
		gsub("^ ", "", strnum)		# remove a starting space
		gsub(" $", "", strnum)		# remove an ending space
		gsub(" ", " </str>\n</wd>\n<wd>\n<str> ", strnum)	# add tags
		strnum = "<str> " strnum " </str>"			# add tags

		# Print
		print strnum > Output
		next
	}
}

(xml == 0) {
	# Verify if the input string is a number
	nombre = $0
	sub(/[\.0-9]*/, "", nombre)

	# Num2Text only numbers! 
	if (length(nombre) == 0)
	{
		strnum = FCT_Convert($0)
		gsub("[  ]+", " ", strnum)	# remove multiple spaces
		gsub("^ ", "", strnum)		# remove a starting space
		print strnum > Output
		next
	}
}

{
	print > Output
}

# ############################## END MAIN ################################# #


