#! /usr/bin/awk -f
# ######################################################################### #
#                                                                           #
# Program Name:  PosTagger.awk                                              #
# Author:        Brigitte Bigi                                              #
# Licence:       GPL                                                        #
# Date:          April, 2007                                                #
# Brief:         POS Tagger, using external lia_tagger toolkit              #
# Help:          PosTagger.awk -h                                           #
#                                                                           #
# ######################################################################### #


# ######################################################################### #
#                                                                           #
# Function Name: FCT_Usage                                                  #
#                                                                           #
# Description:                                                              #
#     Print the usage and exit the program                                  #
#                                                                           #
# ######################################################################### #
function FCT_Usage ()
{
        printf "Usage: PosTagger.awk [options]\n\n"
        printf "Where options include:\n"
	printf "  -l           call LIA tagger + lemmas instead of tagger only\n"
        printf "  -o filename  write the output to this file (default: stdout)\n"
        printf "  -h           print this help\n"
        printf "\n"
        printf " input XML:  stdin\n"
        printf " output XML: stdout\n"
        exit
}

# ############################# BEGIN MAIN ################################ #

BEGIN {
        Output      = "/dev/stdout"     # output file name
	tagger      = "$LIA_TAGG/bin/lia_unmotparligne | $LIA_TAGG/script/lia_tagg+reacc"
	lemma       = 0

	# Extract command line, with options
        for (i=1; i<ARGC; i++)
        {
                if (ARGV[i] == "-h")
                        FCT_Usage()
                else
                if (ARGV[i] == "-l")
                {
			tagger = "$LIA_TAGG/bin/lia_unmotparligne | $LIA_TAGG/script/lia_tagg+lemm"
			lemma  = 1
                        i++
                }
                else
                if (ARGV[i] == "-o")
                {
                        Output = ARGV[i+1]
                        i++
                }
                else
                if (ARGV[i] ~ /^\-/)
                {
                        printf "invalid option -- %s\n", ARGV[i] > "/dev/stderr"
                        FCT_Usage()
                }
        }
	delete ARGV
	ARGC = 1
}


/<elts[ >]/ {
	elt = 1
	str = "<s> "
	print
	next
}


/<\/elts>/ {
	str = str "</s>"

	# Use an external tagger and keep result

	# ensure the "echo" will work, not depending which shell is running!
	str = gensub(/(.)/, "\\\\\\1", "g", str)
	command = ("echo " str " | " tagger)

	# The tagger output is one word by line, between <s> and </s>
	# 1st column is the word, 2nd column is the tag, 3rd is the lemma
	i = 1
	while ((command | getline) > 0)
	{
		if (!match($0, /<[\/]?s>/))
		{
			# Process output of command
			TabStr[i] = $1
			TabTag[i] = $2
			if (lemma == 1)
				TabLemma[i] = $3
			i ++
		}
	}
        close(command)

	if (i != elt)
	{
		print "ERROR. DEBUG IS:"
		print "the input string:"
		print str
		print "the values:"
		print "i =",i,"and elt =",elt #> "/dev/stderr"
		for (i=1; i<elt; i++)
			print i,TabStr[i]
		exit
	}

	for (i=1; i<elt; i++)
	{
		print TabWd[i] > Output
		print "<str>",TabStr[i],"</str>" > Output
		print "<pos>",TabTag[i],"</pos>" > Output
		if (lemma == 1)
			print "<lem>",TabLemma[i],"</lem>" > Output
		print "</wd>" > Output
	}
	print "</elts>" > Output
	# clean!
	delete TabWd
	delete TabStr
	delete TabTag
	delete TabLemma
	next
}

/<wd[ >]/ {
	# keep tag into TabElt (in case wd has num)
	s = index($0, ">")
	w = substr($0, 0, s+1)
	TabWd[elt] = w 
	###elt++ -> NO! in case of <str> is corrupted, <wd> is cancelled!!!
	next
}

/<\/wd>/ {
	next
}

/<str>/ {
	if (elt > 0)
	{
	        # Get the word, without tags
	        s = index($0, ">")                              # start: 1st >
        	e = index(substr($0, s+1, length($0)), "<")     # end: 1st < after start
	        w = substr($0, s+1, e-1)                        # word: $0 from start to end
        	gsub(/ /, "", w)                                # remove spaces

		if (length(w)>0)
		{
			str = str w " "
			elt++	# <str> is a valid word!
		}
	}
	else
	{
		print "parsing error" > "/dev/stderr"
		exit 1
	}
	next
}

{
	print > Output
}
