#! /usr/bin/gawk -f
# ######################################################################### #
#                                                                           #
# File:     Replace.awk                                                     #
# Authors:  Brigitte Bigi                                                   #
# Date:     April, 2007                                                     #
# Licence:  GPL                                                             #
# Brief:    Replace elements from a list                                    #
# Help:     Replace.awk -h                                                  #
#                                                                           #
# ######################################################################### #


# ######################################################################### #
#                                                                           #
#  Function FCT_Load                                                        #
#                                                                           #
#  Description: Load 2 columns from a file                                  #
#                                                                           #
#  Parameters:                                                              #
#       file is the file name                                               #
#                                                                           #
#  Output:                                                                  #
#       TAB_Replace                                                         #
#                                                                           #
#  Return value:                                                            #
#       integer value: the number of entries in TAB_Replace                 #
#                                                                           #
# ######################################################################### #
function FCT_Load ( file )
{
        _ind = 0
        while ( getline <  file  )
        {
                TAB_Replace[$1]=$2
                _ind++
        }
        return _ind
}
# ######################################################################### #


# ######################################################################### #
#                                                                           #
# Function Name: FCT_Usage                                                  #
#                                                                           #
# Description:                                                              #
#     Print the usage and exit the program                                  #
#                                                                           #
# ######################################################################### #
function FCT_Usage ()
{
        printf "Usage: Replace.awk [options]\n\n"
        printf "Where options include:\n"
	printf "  -V filename  a two columns file\n"
	printf "  -l lang      language FR|CN|EN (default: none)\n"
	printf "  -o filename  write the output to this file (default: stdout)\n"
        printf "  -h	print  this help\n"
	printf "\n"
	printf " input text:  stdin\n"
	printf " output text: stdout (except if -o)\n"
        exit
}


# ######################################################################### #
#                                                                           #
# Function Name: FCT_CommandLine                                            #
#                                                                           #
# Description:                                                              #
#     Get arguments from the command line                                   #
#                                                                           #
# ######################################################################### #
function FCT_CommandLine(argc, argv)
{
        entry = 0
        for (i=1; i<argc; i++)
        {
                if (argv[i] == "-h")
                        FCT_Usage()
                else
                if (argv[i] == "-V")
                {
                        tabsize = FCT_Load(argv[i+1])
                        i++
                }
                else
                if (argv[i] == "-o")
                {
                        Output = argv[i+1]
                        i++
                }
                else
                if (argv[i] == "-l")
                {
                        lang = argv[i+1]
                        i++
                }
                else
		if (argv[i] ~ /^\-/)
		{
			printf "invalid option -- %s\n", argv[i] > "/dev/stderr"
			FCT_Usage()
		}
        }
}


# ############################# BEGIN MAIN ################################ #

BEGIN{
	# Need, at least, the -V option
        if ( ARGC == 1 ) FCT_Usage()

	lang        = none		# default language
        Output      = "/dev/stdout"	# output file name
	tabsize     = 0			# vocabulary size
        FCT_CommandLine(ARGC, ARGV)	# extract parameters
        delete ARGV
        ARGC=1
}

# New entry
/<str[ >]/ {
	# Get the element
        s = index($0, ">")				# start: 1st >
        e = index(substr($0, s+1, length($0)), "<")	# end: 1st < after start
        w = substr($0, s+1, e-1)			# word: $0 from start to end
        gsub(/ /, "", w)				# remove spaces
        stag = substr($0, 0, s)
        etag = "</str>"

        # Empty element? ignore!
        if (length(w) == 0) next


	# -------
	# NUMBERS and '
	# -------
	if (lang == "FR")
	{
		# Replace "1,1" by "1 virgule 1"
		w = gensub(/([0-9]),([0-9])/, "\\1_virgule_\\2", "g", w)
	}
	else
	if (lang == "CN")
	{
		# Replace "1.1" by "1 point 1"        
		# Dont forget to put the word "point" and its translation in the dico!
		w = gensub(/([0-9])\.([0-9])/, "\\1_point_\\2", "g", w)
	}
	else
	if (lang == "EN")
	{
		# Replace "1,1" by "11"
		w = gensub(/([0-9]),([0-9])/, "\\1\\2", "g", w)
		# Replace "1.1" by "1 point 1"
		w = gensub(/([0-9])\.([0-9])/, "\\1_point_\\2", "g", w)
		# Pbs with '
		w = gensub(/'([a-zA-Z][a-zA-Z,.?])/, "\"\\1", "g", w)
		w = gensub(/([a-zA-Z][a-zA-Z,.?])'/, "\\1\"", "g", w)
	}

	# ----
	# DICO
	# ----
	for (c in TAB_Replace)
	{
		if (match(w,c))
			gsub(c,TAB_Replace[c],w)
	}

	# Print the new element
        nb = split(w, Lst_Elt, "_")
        for (i=1; i<=nb; i++)
	{
                if (length(Lst_Elt[i]) > 0)
		{
                        print stag,Lst_Elt[i],etag > Output
			print "</wd>" > Output
			print "<wd>" > Output
		}
	}
	next
}

{
	print > Output
}
