#! /usr/bin/gawk -f
# ######################################################################### #
#                                                                           #
# Program Name:  SplitOrig2WdKH.awk                                         #
# Author:        Brigitte Bigi, Sopheap SENG                                #
# Licence:       GPL                                                        #
# Date:          November, 2007                                             #
# Brief:         Split the orig sentence to create clusters of khmer chars  #
# Help:          SplitOrig2WdKH.awk -h                                      #
#                                                                           #
# ######################################################################### #


# ######################################################################### #
#                                                                           #
# Function Name: FCT_Usage                                                  #
#                                                                           #
# Description:                                                              #
#     Print the usage and exit the program                                  #
#                                                                           #
# ######################################################################### #
function FCT_Usage ()
{
        printf "Usage: SplitOrig2WdKH.awk [options]\n\n"
        printf "Where options include:\n"
	printf "  -e id        give a specific id to elts (default: elt)\n"
        printf "  -o filename  write the output to this file (default: stdout)\n"
        printf "  -h           print this help\n"
        printf "\n"
        printf " input text:  stdin\n"
        printf " output text: stdout\n"
        exit
}


# ######################################################################### #
#                                                                           #
# Function Name: FCT_CommandLine                                            #
#                                                                           #
# Description:                                                              #
#     Get arguments from the command line                                   #
#                                                                           #
# ######################################################################### #
function FCT_CommandLine(argc, argv)
{
        entry = 0
        for (i=1; i<argc; i++)
        {
                if (argv[i] == "-h")
                        FCT_Usage()
                else
                if (argv[i] == "-e")
                {
                        EltsID = argv[i+1]
                        i++
                }
                else
                if (argv[i] == "-o")
                {
                        Output = argv[i+1]
                        i++
                }
                else
                if (argv[i] ~ /^\-/)
                {
                        printf "invalid option -- %s\n", argv[i] > "/dev/stderr"
                        FCT_Usage()
                }
        }
}


# ############################# BEGIN MAIN ################################ #


BEGIN {
	EltsID      = "elt"		# elts ID
        Output      = "/dev/stdout"     # output file name
        FCT_CommandLine(ARGC, ARGV)     # extract parameters
        delete ARGV
        ARGC=1

	orig = ""
	inorig = 0
}

/<[\/]?sent>/ {
	next
}

/<orig>/ {
	# inside the original string is now true for this line
	# and the next ones... until </orig>
	inorig = 1
	next
}

/<\/orig>/ {
	# not an empty string... print orig and elts
	if (length(orig) > 0)
	{
		print "<sent>" > Output
		print "<orig>" > Output
		print orig     > Output
		print "</orig>" > Output
		printf "<elts id=\"%s\">\n",EltsID > Output

		# Make some cleaning
		orig = gensub( /([កខគឃងចឆជឈញដឋឌឍណតថទធនបផពភមយរលវសហឡអ]|[ឥឦឧឪឳឩឰឱឮឭឬឫ])([៊៉]*[្][កខគឃងចឆជឈញដឋឌឍណតថទធនបផពភមយរលវសហឡអ][្][កខគឃងចឆជឈញដឋឌឍណតថទធនបផពភមយរលវសហឡអ][៊៉]*[ាិីឹឺុូួើឿៀេែៃោៅំះៈ]*[៍័៏៌់]*|[៊៉]*[្][កខគឃងចឆជឈញដឋឌឍណតថទធនបផពភមយរលវសហឡអ][៊៉]*[ាិីឹឺុូួើឿៀេែៃោៅំះៈ]*[៍័៏៌់]*|[៊៉]*[ាិីឹឺុូួើឿៀេែៃោៅំះៈ]*[៍័៏៌់]*)/ , "_&_", "g", orig)


		# Replace ^M by space
        	gsub(/\r/, "_", orig)

	        # Replace -- by only one _
        	while ( match(orig,/--/) )
                	gsub(/--/, "_", orig)

        	gsub(/ /, "_", orig)	        

	        # Add spaces
        	orig = gensub(/([$+>^@}{~|=])([[:alnum:]])/, "_\\1_\\2", "g", orig)

	        # Split numbers if sticked to chars
        	orig = gensub(/([[:digit:]])([^[:digit:]])/, "\\1_\\2", "g", orig)
	        orig = gensub(/([^[:digit:]])([[:digit:]])/, "\\1_\\2", "g", orig)
	
		# Split numbers if sticked to chars - Add Khmer digit
		#orig = gensub(/([០១២៣៤៥៦៧៨៩])([^០១២៣៤៥៦៧៨៩])/, "\\1_\\2", "g", orig)
		#orig = gensub(/([^០១២៣៤៥៦៧៨៩])([០១២៣៤៥៦៧៨៩])/, "\\1_\\2", "g", orig)

	        # Split dots if sticked to a word
        	orig = gensub(/ \.([[:alpha:]-])/, "._\\1", "g", orig)
		orig = gensub(/^\.([[:alpha:]-])/, "._\\1", "g", orig)

	        # Stick numbers 
	        orig = gensub(/([[:digit:]])[ ]+([[:digit:]])/, "\\1\\2", "g", orig)
		
		# Stick khmer numbers
	        #orig = gensub(/([០១២៣៤៥៦៧៨៩])[_]+([០១២៣៤៥៦៧៨៩])/, "\\1\\2", "g", orig)

		# and voila!
		gsub(/__/, "_", orig)
	        gsub(/^_/, "", orig)
        	gsub(/_$/, "", orig)

		nb = split(orig, TAB_Str, "_")
		for (i=1; i<=nb; i++)
			printf "<wd>\n<str> %s </str>\n</wd>\n", TAB_Str[i] > Output
		print "</elts>" > Output
		print "</sent>" > Output
		orig = ""
		delete TAB_Str
	}
	inorig=0
	next
}


(inorig==1) {
	# inside orig
	# do some cleaning... 
	gsub("[  ]+", " ")
	gsub("^ ", "")
	gsub("^	", "")

	# and remember it!
	if (length($0) > 0)
		orig = orig $0

	# dont print... get next line
	next
}

{
	print > Output
}
