#! /usr/bin/gawk -f
# ######################################################################### #
#                                                                           #
# File:     Stick.awk                                                       #
# Authors:  Brigitte Bigi, Viet Bac Le, Sopheap SENG                        #
# Date:     Nov, 2007                                                       #
# Licence:  GPL                                                             #
# Brief:    Make groups by longest matching algorithm                       #
# Help:     Stick.awk -h                                                    #
#                                                                           #
# ######################################################################### #
  
 
# ######################################################################### #
#                                                                           #
#  Function FCT_LoadVocab                                                   #
#                                                                           #
#  Description: Load a vocabulary from a file                               #
#                                                                           #
#  Parameters:                                                              #
#       file is the file name of the vocabulary                             #
#                                                                           #
#  Output:                                                                  #
#       TAB_Voc is a string array                                           #
#                                                                           #
#  Return value:                                                            #
#       integer value: the number of words (TAB_Voc size)                   #
#                                                                           #
# ######################################################################### #
function FCT_LoadVocab ( file )
{
	_ind = 0
	while ( getline <  file  )
	{
		TAB_Voc[$0]=_ind
		_ind++
	}
	return _ind
}
# ######################################################################### #



# ######################################################################### #
#                                                                           #
#  Function FCT_IsOOV                                                       #
#                                                                           #
#  Return value:                                                            #
#       boolean value: 1 if word is in TAB_Voc, 0 else                      #
#                                                                           #
# ######################################################################### #
function FCT_IsOOV ( word )
{
	if (TAB_Voc[word] != "") return 0
	return 1
}


# ######################################################################### #
#                                                                           #
# fonction FCT_Stick                                                        #
#                                                                           #
# ######################################################################### #
function FCT_Stick()
#	- if a stick is possible : print the stick and remove 
#	  all sticked words from the list
#	- else, the older word is saved and removed from the list
{
	# i is used to move in LST_Words
	i = nbw

	# i_ok is the index of the end of the sticked word (if any)
	i_ok = 1

	while ( i >= 1 && i_ok == 1)
	{
		# Try to stick all tokens
		token = LST_Words[1]
		if (i > 1)
		{
			for (k = 2; k <= i; k++)
				token = token attachement LST_Words[k]
			# Found a stick in the vocab
			if ( FCT_IsOOV(token) == 0 ) 
				i_ok = i
		}	
		i--
	}

	########print stag, token, etag > Output
	strout = strout " " token

	# shift LST_Words
	for (k=i_ok+1; k<=nbw; k++)
		LST_Words[k-i_ok] = LST_Words[k]
	nbw = nbw-i_ok
}


# ######################################################################### #
#                                                                           #
# Function Name: FCT_Usage                                                  #
#                                                                           #
# Description: Print the usage and exit the program                         #
#                                                                           #
# ######################################################################### #
function FCT_Usage ()
{
        printf "Usage: Stick.awk [options] \n\n"
        printf "Where options include:\n"
        printf "  -V filename  read vocabulary from this file\n"
	printf "  -a char      char used to join words in a stick\n"
	printf "  -s number    max stick size (default: 5)\n"
	printf "  -o filename  write the output to this file (default: stdout)\n"
	printf "  -e id        stick on these id elts (default: elt) \n"
	printf "  -t id        dont modify the elts and create a new elts serie with this id (default: to) \n"
        printf "  -h           print this help\n"
        printf "\n"
        printf " input text:  stdin\n"
        printf " output text: stdout (except if -o)\n"
        exit
}


# ######################################################################### #
#                                                                           #
# Function Name: FCT_CommandLine                                            #
#                                                                           #
# Description: Gets arguments from the command line                         #
#                                                                           #
# ######################################################################### #
function FCT_CommandLine(argc, argv)
{
        for (i=1; i<ARGC; i++)
        {
                if (argv[i] == "-h")
                        FCT_Usage()
                else
                if (argv[i] == "-V")
                {
                        vocabsize = FCT_LoadVocab(argv[i+1])
                        i++
                }
                else
                if (argv[i] == "-a")
                {
                        attachement = argv[i+1]
                        i++
                }
                else
                if (argv[i] == "-s")
                {
                        MaxS = argv[i+1]
                        i++
                }
                else
                if (argv[i] == "-o")
                {
                        Output = argv[i+1]
                        i++
                }
                else
                if (argv[i] == "-e")
                {
                        id = argv[i+1]
                        i++
                }
                else
                if (argv[i] == "-t")
                {
                        newid = argv[i+1]
                        i++
                }
                else
                if (argv[i] ~ /^\-/)
                {
                        printf "invalid option -- %s\n", argv[i] > "/dev/stderr"
                        FCT_Usage()
                }
        }
}

# ######################################################################### #
#                                                                           #
# Function Name: FCT_Print                                                  #
#                                                                           #
# Description: Format and print the new sentence                            #
#                                                                           #
# ######################################################################### #
function FCT_Print ()
{
	# Format the output (see dtd!)
	gsub("[  ]+", " ", strout)      # remove multiple spaces
	gsub("^ ", "", strout)          # remove a starting space
	gsub(" $", "", strout)          # remove an ending space
	gsub(" ", " </str>\n</wd>\n<wd>\n<str> ", strout)	# add tags
	strout = "<wd>\n<str> " strout " </str>\n</wd>"		# add tags

	# Print
	print strout > Output
}

# ############################# BEGIN MAIN ################################ #


BEGIN {
	if ( ARGC == 1 )
                FCT_Usage()

	MaxS        = 5		        # Max stick size
	Output      = "/dev/stdout"	# output file name
	attachement = ""		# stick character
        vocabsize = 0			# vocabulary size
	nbw = 0				# number of words placed in the list
	id="elt"			# id of elts that Stick will work with (default elt)
	newid="to"			# id of news elts
        FCT_CommandLine(ARGC, ARGV)	# extract parameters
        delete ARGV
        ARGC=1

	if (newid!="to")
		keepOrigine=1		# keep original elts or not
	else
		keepOrigine=0
}

# new elements 
/<elts[ >]/ {
	if (index($0, id) != 0)
	# find the elts named id,
	{		
		inelts = 1
		strout = " "
		nbw    = 0
		if (keepOrigine == 1) print > Output
	}
	next
}

# end of elements: print!
/<\/elts>/ {
	inelts = 0
	while ( nbw > 0 )
		FCT_Stick()

	# Print
	if (keepOrigine == 1) print > Output

	if (newid != "to")
	# if newid is given so != to, Add news elts named id
		print "<elts id=\"" newid "\">" > Output
	else
		print "<elts id=\"" id "\">" > Output
	FCT_Print()
	print "</elts>" > Output

	next
}

/<str[ >]/ {
        # Get the word
        s = index($0, ">")				# start: 1st >
        e = index(substr($0, s+1, length($0)), "<")	# end: 1st < after start
        w = substr($0, s+1, e-1)			# word: $0 from start to end
        gsub(/ /, "", w)				# remove spaces
	stag = substr($0, 0, s)
	etag = "</str>"

	# Empty word ?
	if (length(w) == 0)
		next

	# the new word is placed in the list
	nbw++
	LST_Words[nbw] = w

	# When the list is full (full means MaxS words), attempt to stick
	if ( nbw == MaxS ) FCT_Stick()

	if (keepOrigine == 1) print > Output 

	next
}

{
	# other tags must NOT be changed
	if (inelts == 0 || keepOrigine == 1) print > Output
}

# ############################## END MAIN ################################## #
