#! /usr/bin/gawk -f
# ######################################################################### #
#                                                                           #
# File:     XML2WordCounts.awk                                              #
# Authors:  Brigitte Bigi                                                   #
# Date:     Nov, 2003                                                       #
# Licence:  GPL                                                             #
# Brief:    Print list of word counts from a xml corpus                     #
# Help:     XML2WordCounts.awk -h                                           #
#                                                                           #
# ######################################################################### #

# ######################################################################### #
#                                                                           #
# Function Name: FCT_Usage                                                  #
#                                                                           #
# Description:                                                              #
#     Print the usage and exit the program                                  #
#                                                                           #
# ######################################################################### #
function FCT_Usage ()
{
        printf "Usage: XML2WordCounts.awk [options]\n\n"
        printf "Where options include:\n"
        printf "  -h	print this help\n"
	printf "\n"
	printf " input text:  stdin\n"
	printf " output text: stdout\n"
        exit
}


# ######################################################################### #
#                                                                           #
# Function Name: FCT_CommandLine                                            #
#                                                                           #
# Description:                                                              #
#     Get arguments from the command line                                   #
#                                                                           #
# ######################################################################### #
function FCT_CommandLine(argc, argv)
{
        entry = 0
        for (i=1; i<argc; i++)
        {
                if (argv[i] == "-h")
                        FCT_Usage()
                else
		if (argv[i] ~ /^\-/)
		{
			printf "invalid option -- %s\n", argv[i] > "/dev/stderr"
			FCT_Usage()
		}
                        break
        }
}


# ############################# BEGIN MAIN ################################ #

BEGIN {
        FCT_CommandLine(ARGC, ARGV)
        delete ARGV
        ARGC=1
}

/<str>/{
        # Get the word
        s = index($0, ">")				# start: 1st >
        e = index(substr($0, s+1, length($0)), "<")	# end: 1st < after start
        w = substr($0, s+1, e-1)			# word: $0 from start to end
        gsub(/ /, "", w)				# remove spaces

	# Create a tab word/count
	TAB_Counts[w]++
}

END {
	# sort tab and print on stdout
	sort = "sort -k 2 -nr"
	for (word in TAB_Counts)
		print word, TAB_Counts[word] | sort
	close(sort)
}

################################ END MAIN ###################################

